@@ -262,7 +262,7 @@ archiveFileLog fLog entryId = do
   logFiles <- findLogFiles (logIdentifier fLog)
   let sorted = sort logFiles
       relevant = filterLogFiles Nothing (Just entryId) sorted \\
-                 filterLogFiles (Just (entryId+1)) (Just entryId) sorted
+                 filterLogFiles (Just entryId) (Just (entryId+1))  sorted
 
   createDirectoryIfMissing True archiveDir
   forM_ relevant $ \(_startEntry, logFilePath) ->
