@@ -78,6 +78,7 @@ module Data.Acid.Remote
     (
     -- * Server/Client
       acidServer
+    , acidServer'
     , openRemoteState
     -- * Authentication
     , skipAuthenticationCheck
@@ -145,7 +146,6 @@ data AcidRemoteException
       deriving (Eq, Show, Typeable)
 instance Exception AcidRemoteException
 
-
 -- | create a 'CommChannel' from a 'Handle'. The 'Handle' should be
 -- some two-way communication channel, such as a socket
 -- connection. Passing in a 'Handle' to a normal is file is unlikely
@@ -224,6 +224,28 @@ acidServer :: SafeCopy st =>
 acidServer checkAuth port acidState
   = withSocketsDo $
     do listenSocket <- listenOn port
+       (acidServer' checkAuth listenSocket acidState) `finally` (cleanup listenSocket)
+    where
+      cleanup socket =
+          do sClose socket
+             case port of
+#if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
+               UnixSocket path -> removeFile path
+#endif
+               _               -> return ()
+
+{- | Works the same way as 'acidServer', but uses pre-binded socket @listenSocket@.
+
+     Can be useful when fine-tuning of socket binding parameters is needed
+     (for example, listening on a particular network interface, IPv4/IPv6 options).
+ -}
+acidServer' :: SafeCopy st =>
+              (CommChannel -> IO Bool) -- ^ check authentication, see 'sharedSecretPerform'
+           -> Socket                   -- ^ binded socket to accept connections from
+           -> AcidState st             -- ^ state to serve
+           -> IO ()
+acidServer' checkAuth listenSocket acidState
+  = do
        let loop = forever $
              do (socket, _sockAddr) <- accept listenSocket
                 let commChannel = socketToCommChannel socket
@@ -232,7 +254,7 @@ acidServer checkAuth port acidState
                                  process commChannel acidState
                             ccClose commChannel -- FIXME: `finally` ?
            infi = loop `catchSome` logError >> infi
-       infi `finally` (cleanup listenSocket)
+       infi
     where
       logError :: (Show e) => e -> IO ()
       logError e = hPrint stderr e
@@ -251,28 +273,24 @@ acidServer checkAuth port acidState
                             then return () -- h (toException e) -- we could log the exception, but there could be thousands of them
                             else throw e
                        ]
-      cleanup socket =
-          do sClose socket
-             case port of
-#if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
-               UnixSocket path -> removeFile path
-#endif
-               _               -> return ()
 
 data Command = RunQuery (Tagged Lazy.ByteString)
              | RunUpdate (Tagged Lazy.ByteString)
              | CreateCheckpoint
+             | CreateArchive
 
 instance Serialize Command where
   put cmd = case cmd of
               RunQuery query   -> do putWord8 0; put query
               RunUpdate update -> do putWord8 1; put update
               CreateCheckpoint ->    putWord8 2
+              CreateArchive    ->    putWord8 3
   get = do tag <- getWord8
            case tag of
              0 -> liftM RunQuery get
              1 -> liftM RunUpdate get
              2 -> return CreateCheckpoint
+             3 -> return CreateArchive
              _ -> error $ "Serialize.get for Command, invalid tag: " ++ show tag
 
 data Response = Result Lazy.ByteString | Acknowledgement | ConnectionError
@@ -304,9 +322,12 @@ process CommChannel{..} acidState
        worker chan (runGetPartial get Strict.empty)
   where worker chan inp
           = case inp of
-              Fail msg      -> throwIO (SerializeError msg)
+              Fail msg _    -> throwIO (SerializeError msg)
               Partial cont  -> do bs <- ccGetSome 1024
-                                  worker chan (cont bs)
+                                  if Strict.null bs then
+                                     return ()
+                                  else
+                                     worker chan (cont bs)
               Done cmd rest -> do processCommand chan cmd; worker chan (runGetPartial get rest)
         processCommand chan cmd =
           case cmd of
@@ -316,7 +337,8 @@ process CommChannel{..} acidState
                                    writeChan chan (liftM Result $ takeMVar result)
             CreateCheckpoint -> do createCheckpoint acidState
                                    writeChan chan (return Acknowledgement)
-
+            CreateArchive -> do createArchive acidState
+                                writeChan chan (return Acknowledgement)
 
 data RemoteState st = RemoteState (Command -> IO (MVar Response)) (IO ())
                     deriving (Typeable)
@@ -407,7 +429,7 @@ processRemoteState reconnect
                  getResponse leftover =
                      do debugStrLn $ "listener: listening for Response."
                         let go inp = case inp of
-                                   Fail msg       -> error msg
+                                   Fail msg _     -> error msg
                                    Partial cont   -> do debugStrLn $ "listener: ccGetSome"
                                                         bs <- ccGetSome cc 1024
                                                         go (cont bs)
@@ -477,7 +499,6 @@ remoteQueryCold rs@(RemoteState fn _shutdown) event
                                remoteQueryCold rs event
          Acknowledgement    -> error "remoteQueryCold got Acknowledgement. That should never happen."
 
-
 scheduleRemoteUpdate :: UpdateEvent event => RemoteState (EventState event) -> event -> IO (MVar (EventResult event))
 scheduleRemoteUpdate (RemoteState fn _shutdown) event
   = do let encoded = runPutLazy (safePut event)
@@ -505,6 +526,11 @@ createRemoteCheckpoint (RemoteState fn _shutdown)
   = do Acknowledgement <- takeMVar =<< fn CreateCheckpoint
        return ()
 
+createRemoteArchive :: RemoteState st -> IO ()
+createRemoteArchive (RemoteState fn _shutdown)
+  = do Acknowledgement <- takeMVar =<< fn CreateArchive
+       return ()
+
 toAcidState :: IsAcidic st => RemoteState st -> AcidState st
 toAcidState remote
   = AcidState { _scheduleUpdate    = scheduleRemoteUpdate remote
@@ -512,6 +538,8 @@ toAcidState remote
               , _query             = remoteQuery remote
               , queryCold          = remoteQueryCold remote
               , createCheckpoint   = createRemoteCheckpoint remote
+              , createArchive      = createRemoteArchive remote
               , closeAcidState     = closeRemoteState remote
               , acidSubState       = mkAnyState remote
               }
+
